import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * Clase que representa el trablero del juego, controla la insercion y carga de
 * los actores como el escenario.
 * 
 * @author Jaime Vazquez Castillo
 * @version 04082010
 */
public class Board  extends World
{
    private static int square = 60;
    private static int x0 = 243;
    private static int y0 = 110;
   
    private int[][] walls;
    private int level;
    private boolean intro;
    private int iState;

 
   /**
     * Constructor for objects of class Board.
     * 
     */
    public Board()
    {    
        super(640, 480, 1);
        intro = true;
        iState = 0;
        
        Dummy title = new Dummy();
        title.setImage("titlescreen.png");
        Dummy b0 = new Dummy();
        b0.setImage("bhelp.png");
        Dummy b1 = new Dummy();
        b1.setImage("bstart.png");
        addObject(title,320,240);
        addObject(b0,163,396);
        addObject(b1,480,397);
        Greenfoot.playSound("tombslide.wav");
    }
    
    /**
     *  Metodo act() es llamado cada vez que el juego corre.
     */
    public void act()
    {
        if(intro)
        {
            List aux = getObjects(Dummy.class);
            
            Dummy daux = (Dummy) aux.get(1);
            if(Greenfoot.mouseClicked(daux))
            {
                switch(iState)
                {
                    case 0 :
                    case 2 :
                    {
                        daux.setImage("expInvisible.png");
                        daux = (Dummy) aux.get(0);
                        daux.setImage("help0.png");
                        daux = (Dummy) aux.get(2);
                        daux.setImage("arrowR.png");
                        daux.setLocation(571,451);
                        iState = 1;
                    }break;
                }
            }
            else
            {
                daux = (Dummy) aux.get(2);
                if(Greenfoot.mouseClicked(daux))
                {
                    switch(iState)
                    {
                        case 0 :
                        case 2 :
                        {
                            intro = false;
                            removeObjects(getObjects(Dummy.class));
                            level = 1;
                            addObject(new Undo(), 73, 152);
                            addObject(new Reset(), 73, 194);
                            addObject(new Floor(), 393, 260);
                            loadLevel(level);
                            createWalls();
                        }break;
                        
                        case 1 :
                        {
                            daux.setImage("bstart.png");
                            daux.setLocation(485,444);
                            daux = (Dummy) aux.get(1);
                            daux.setImage("arrowL.png");
                            daux.setLocation(87,446);
                            daux = (Dummy) aux.get(0);
                            daux.setImage("help1.png");
                            iState = 2;
                        }
                    }
                }
            }            
            
            
            
        }
        else//game started...
        {
            List aux = getObjects(Stairs.class);
            
            if(!aux.isEmpty())
            {
                Stairs stairs = (Stairs) aux.get(0);
            
                if(stairs.escaped())
                {
                    level++;
                    removeObjects(getObjects(Enemy.class));
                    removeObjects(getObjects(Explorer.class));
                    removeObjects(getObjects(Scenario.class));
                    loadLevel(level);
                    if(level!=11) createWalls();
                }
            }
        }
    }
    
    /**
     * Inicializa las paredes. Borra si existen.
     */
    private void createWalls()
    {
        removeObjects(getObjects(WallV.class));
        removeObjects(getObjects(WallH.class));
        
        //wallH 0,0 x=245 y=137
        //wallV 0,0 x=276 y=107
        int cond;
        for(int i=0; i<6; i++)
        {
            for(int j=0; j<6; j++)
            {
                cond = walls[i][j];
                switch(cond)
                {
                    case 1 :
                    {
                        addObject(new WallV(),276+(i*square),107+(j*square));
                    }
                    break;
                    
                    case 2 :
                    {
                        addObject(new WallH(),245+(i*square),137+(j*square));
                    }
                    break;
                    
                    case 3 :
                    {
                        addObject(new WallV(),276+(i*square),107+(j*square));
                        addObject(new WallH(),245+(i*square),137+(j*square));
                    }break;
                }
            }
        }
    }
    
    /**
     * Laberinto - Paredes - Posiciones
     * Carga los datos del laberinto segun el nivel.
     * @param level nivel del cual se cargaran los daros.
     */
    private void loadLevel(int level)
    {
        Explorer auxE;
        Mummy auxM;
        int eX,eY,mX,mY,sX,sY,sBX,sBY;
        eX=eY=mX=mY=sX=sY=sBX=sBY=0;
        
        switch(level) 
        {
            case 1 :
            {
                walls = new int[][] {{0,0,1,0,2,0},{0,0,0,3,0,0},{2,3,0,0,0,0},
                         {0,0,0,0,0,0},{0,1,2,0,2,0},{0,2,0,2,2,0}};
                      /*{0,0,2,0,0,0,
                         0,0,3,0,1,2,
                         1,0,0,0,2,0,
                         0,3,0,0,0,2,
                         2,0,0,0,2,2,
                         0,0,0,0,0,0
                         
                         0,0,2,4,0,3}*/
                         mX = 0;
                         mY = 0;
                         eX = 2;
                         eY = 4;
                         sX = 0;
                         sY = 3;
            }
            break;
            
            case 2 :
            {
                walls = new int[][] {{1,1,1,0,0,0},{1,0,2,0,2,0},{2,3,2,2,0,0},
                         {0,1,0,1,3,0},{0,0,1,1,0,0},{2,0,2,0,0,0}};
                      /*{1,1,2,0,0,2,
                         1,0,3,1,0,0,
                         1,2,2,0,1,2,
                         0,0,2,1,1,0,
                         0,2,0,3,0,0,
                         0,0,0,0,0,0,
                         
                         5,3,2,2,5,5}*/
                         mX = 5;
                         mY = 3;
                         eX = 2;
                         eY = 2;
                         sX = 5;
                         sY = 5;
            }
            break;
            
            case 3 :
            {
                walls = new int[][] {{0,0,1,0,0,0},{0,0,2,0,2,0},{2,1,3,0,2,0},
                         {1,1,0,0,3,0},{0,2,0,0,0,0},{0,0,0,0,0,0}};
                      /*{0,0,2,1,0,0,
                         0,0,1,1,2,0,
                         1,2,3,0,0,0,
                         0,0,0,0,0,0,
                         0,2,2,3,0,0,
                         0,0,0,0,0,0,
                         
                         4,5,1,3,5,0}*/
                         mX = 4;
                         mY = 5;
                         eX = 1;
                         eY = 3;
                         sX = 5;
                         sY = 0;
            }
            break;
            
            case 4 :
            {
             walls = new int[][] {{0,0,0,2,2,0},{2,3,1,0,0,0},{0,0,0,0,0,0},
                         {0,2,1,0,0,1},{1,1,1,1,0,0},{0,0,0,0,0,0}};
                      /*{0,2,0,0,1,0,
                         0,3,0,2,1,0,
                         0,1,0,1,1,0,
                         2,0,0,0,1,0,
                         2,0,0,0,0,0,
                         0,0,0,1,0,0,
                         
                         4,4,2,1,0,5}*/
                         mX = 4;
                         mY = 4;
                         eX = 2;
                         eY = 1;
                         sX = 0;
                         sY = 5;
            }
            break;

            case 5 :
            {
            walls = new int[][] {{0,1,1,1,0,0},{0,2,0,0,0,0},{0,2,1,2,2,0},
                         {0,2,2,0,2,0},{2,0,0,0,0,0},{0,0,2,0,0,0}};
                      /*{0,0,0,0,2,0,
                         1,2,2,2,0,0,
                         1,0,1,2,0,2,
                         1,0,2,0,0,0,
                         0,0,2,2,0,0,
                         0,0,0,0,0,0,
                         
                         4,5,0,0,3,5}*/
                         mX = 4;
                         mY = 5;
                         eX = 0;
                         eY = 0;
                         sX = 3;
                         sY = 5;
            }
            break;
            
            case 6 :
            {
            walls = new int[][] {{0,0,0,1,0,0},{2,3,2,1,2,0},{0,2,0,2,2,1},
                         {2,2,0,0,2,0},{0,0,3,0,2,0},{0,0,0,2,0,0}};
                      /*{0,2,0,2,0,0,
                         0,3,2,2,0,0,
                         0,2,0,0,3,0,
                         1,1,2,0,0,2,
                         0,2,2,2,2,0,
                         0,0,1,0,0,0,
                         
                         1,3,1,1,2,5}*/
                         mX = 1;
                         mY = 3;
                         eX = 1;
                         eY = 1;
                         sX = 2;
                         sY = 5;
            }
            break;
            
            case 7 :
            {
            walls = new int[][] {{2,2,1,0,2,0},{0,1,1,2,0,0},{1,0,0,2,0,0},
                         {0,0,0,1,0,1},{2,0,3,0,3,0},{2,0,0,0,0,0}};
                      /*{2,0,1,0,2,2,
                         2,1,0,0,0,0,
                         1,1,0,0,3,0,
                         0,2,2,1,0,0,
                         2,0,0,0,3,0,
                         0,0,0,1,0,0,
                         
                         5,5,1,0,5,4}*/
                         mX = 5;
                         mY = 5;
                         eX = 1;
                         eY = 0;
                         sX = 5;
                         sY = 4;
            }
            break;
            
            case 8 :
            {
            walls = new int[][] {{2,2,0,2,0,0},{0,3,0,3,1,0},{0,0,0,0,0,1},
                         {2,3,2,1,1,0},{0,1,0,0,1,0},{0,2,0,0,2,0}};
                      /*{2,0,0,2,0,0,
                         2,3,0,3,1,2,
                         0,0,0,2,0,0,
                         2,3,0,1,0,0,
                         0,1,0,1,1,2,
                         0,0,1,0,0,0,
                         
                         2,3,2,2,0,4}*/
                         mX = 2;
                         mY = 3;
                         eX = 2;
                         eY = 2;
                         sX = 0;
                         sY = 4;
            }
            break;
            
            case 9 :
            {
            walls = new int[][] {{0,0,0,0,0,1},{2,2,1,2,0,0},{0,0,2,3,2,1},
                         {2,0,1,2,0,1},{0,0,1,1,3,0},{2,0,0,0,0,0}};
                      /*{0,2,0,2,0,2,
                         0,2,0,0,0,0,
                         0,1,2,1,1,0,
                         0,2,3,2,1,0,
                         0,0,2,0,3,0,
                         1,0,1,1,0,0,
                         
                         5,5,0,1,5,5}*/
                         mX = 5;
                         mY = 5;
                         eX = 0;
                         eY = 1;
                         sX = 5;
                         sY = 5;
            }
            break;
            
            case 10 :
            {
            walls = new int[][] {{1,2,0,0,0,0},{0,0,2,0,0,0},{0,0,3,0,1,0},
                         {0,0,0,0,1,0},{1,2,0,0,2,0},{0,0,2,0,0,0}};
                      /*{1,0,0,0,1,0,
                         2,0,0,0,2,0,
                         0,2,3,0,0,2,
                         0,0,0,0,0,0,
                         0,0,1,1,2,0,
                         0,0,0,0,0,0,
                         
                         5,1,2,1,0,0}*/
                         mX = 5;
                         mY = 1;
                         eX = 2;
                         eY = 1;
                         sX = 0;
                         sY = 0;
            }
            break;
            
            case 11 :
                removeObjects(getObjects(Button.class));
                Floor f = (Floor) getObjects(Floor.class).get(0);
                f.setImage("beach.png");
                repaint();
                Greenfoot.stop();
            break;
        }
        
        if(level!=11)
        {
            
        auxE = new Explorer(eX,eY,walls);
        auxM = new Mummy(mX,mY,walls, auxE);
        if(sX == 0)
        {
            sBX = 184; sBY = 100+(sY*square);
        }
        else if(sX == 5)
        {
            sBX = 597; sBY = 100+(sY*square);
        }
        else if(sY == 0)
        {
            sBX = 241+(sX*square); sBY = 47;
        }
        else if(sY == 5)
        {
            sBX = 241+(sX*square); sBY = 473;
        }
        addObject(new Stairs(sX, sY, auxE, auxM), sBX, sBY);
        addObject(auxE, x0+(eX*square), y0+(eY*square));
        addObject(auxM, x0+(mX*square), y0+(mY*square));
        
        }
    }
}
