import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.ArrayList;
import java.util.List;

/**
 * Clase que representa al explorador, controla el movimiento a traves del teclado,
 * asi como la animacion.
 * 
 * @author Jaime Vazquez Castillo 
 * @version 04082010
 */
public class Explorer  extends Actor
{
    private static String[] imgU = {"expUR0.png", "expUR1.png", "expUR0.png",
        "expUN.png", "expUL0.png", "expUL1.png", "expUL0.png", "expUN.png"};
    private static String[] imgD = {"expDR0.png", "expDR1.png", "expDR0.png",
        "expDN.png", "expDL0.png", "expDL1.png", "expDL0.png", "expDN.png"};
    private static String[] imgR = {"expRR0.png", "expRR1.png", "expRR0.png",
        "expRN.png", "expRL0.png", "expRL1.png", "expRL0.png", "expRN.png"};
    private static String[] imgL = {"expLR0.png", "expLR1.png", "expLR0.png",
        "expLN.png", "expLL0.png", "expLL1.png", "expLL0.png", "expLN.png"};
    private static String[] soundW = {"expwalka.wav", "expwalkb.wav"};
        
    private static int animDelay = 2;
    private static int animPixel = 4;
    private static int idleCount = 40;
    private boolean sound = true;
    
    private ArrayList<Integer> stepsX;
    private ArrayList<Integer> stepsY;
    private boolean active;
    private int[][] walls;
    
    /**
     * Constructor con el valor de ubicacion inicial.
     * @param boardX coordenada en x con respecto al tablero.
     * @param boardY coordenada en y con respecto al tablero.
     * @param w arreglo bidimensional con la informacion acerca de la ubicacion de las paredes.
     */
    public Explorer(int boardX, int boardY, int[][] w)
    {
        stepsX = new ArrayList<Integer>();
        stepsY = new ArrayList<Integer>();
        stepsX.add(0,boardX);
        stepsY.add(0,boardY);
        walls = w;
        active = true;
    }
    
    /**
     * Act - Si esta activo, el explorador se movera con respecto al teclado, 
     * si queda sin movimiento se anima.
     * This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        if(active)
        {
           if(idleCount<=31)
           {
               setImage("light" + idleCount + ".png");
               Greenfoot.delay(2);
           }
           if(idleCount>=500) idleCount = 0;
           idleCount++;
           
           
            if(Greenfoot.isKeyDown("down"))
            {
                if(canMoveDown())
                {
                    moveDown();
                    stepsX.add(0,x());
                    stepsY.add(0,y()+1);
                    moveEnemy();
                    idleCount = 40;
                }
                
            }
            
            if(Greenfoot.isKeyDown("up"))
            {
                if(canMoveUp())
                {
                    moveUp();
                    stepsX.add(0,x());
                    stepsY.add(0,y()-1);
                    moveEnemy();
                    idleCount = 40;
                }
            }
            
            if(Greenfoot.isKeyDown("right"))
            {
                if(canMoveRight())
                {
                    moveRight();
                    stepsX.add(0,x()+1);
                    stepsY.add(0,y());
                    moveEnemy();
                    idleCount = 40;
                }
            }
            
            if(Greenfoot.isKeyDown("left"))
            {
                if(canMoveLeft())
                {
                    moveLeft();
                    stepsX.add(0,x()-1);
                    stepsY.add(0,y());
                    moveEnemy();
                    idleCount = 40;
                }
            }
            
            if(Greenfoot.isKeyDown("space"))
            {
            
                stepsX.add(0,x());
                stepsY.add(0,y());
                moveEnemy();
            }
        }
    }
    
    /**
     * Animacion de la luz para el explorador.
     */
    private void light()
    {
        for(int i=0; i<=31; i++)
        {
            setImage("light" + i + ".png");
            Greenfoot.delay(2);
        }
    }
    
    /**
     * Verifica si el explorador puede moverse hacia abajo.
     * 1 = pared este : 2 = pared sur : 3 = ambas
     * @return verdadero o falso segun pueda moverse.
     */
    private boolean canMoveDown()
    {
        if(y() < 5 ) //si no esta en el borde de abajo...
        if(walls[x()][y()] != 2 && walls[x()][y()] != 3) //si no hay paredes que impidan...
            return true;
        
        return false;
    }
    
    /**
     * Verifica si el explorador puede moverse hacia arriba.
     * 1 = pared este : 2 = pared sur : 3 = ambas
     * @return verdadero o falso segun pueda moverse.
     */
    private boolean canMoveUp()
    {
        if(y() > 0 ) //si no esta en el borde de arriba...
        if(walls[x()][y()-1] != 2 && walls[x()][y()-1] != 3) //si no hay paredes que impidan...
            return true;
        
        return false;
    }
    
    /**
     * Verifica si el explorador puede moverse hacia la derecha.
     * 1 = pared este : 2 = pared sur : 3 = ambas
     * @return verdadero o falso segun pueda moverse.
     */
    private boolean canMoveRight()
    {
        if(x() < 5 ) //si no esta en el borde de la derecha...
        if(walls[x()][y()] != 1 && walls[x()][y()] != 3) //si no hay paredes que impidan...
            return true;
        
        return false;
    }
    
    /**
     * Verifica si el explorador puede moverse hacia la derecha.
     * 1 = pared este : 2 = pared sur : 3 = ambas
     * @return verdadero o falso segun pueda moverse.
     */
    private boolean canMoveLeft()
    {
        if(x() > 0 ) //si no esta en el borde de la izquierda...
        if(walls[x()-1][y()] != 1 && walls[x()-1][y()] != 3) //si no hay paredes que impidan...
            return true;
        
        return false;
    }
    
    /**
     * Avisa a los enemigos que se mueven despues del explorador.
     */
    private void moveEnemy()
    {
        List enemy = getWorld().getObjects(Enemy.class);
        Enemy aux;
        for(Object en : enemy)
        {
            aux = (Enemy) en;
            aux.move();
        }
    }
    
    /**
     * Mueve al explorador un paso hacia abajo, incluye animacion.
     */
    public void moveDown()
    {
        int img=0;
        if(sound) Greenfoot.playSound(soundW[Greenfoot.getRandomNumber(2)]);
        for(int i=0; i<(60/animPixel); i++)
        {
            setLocation(getX(),getY()+animPixel);
            setImage(imgD[img++]);
            if(img==8) img = 0;
            Greenfoot.delay(animDelay);
        }
        setImage(imgD[7]);
    }
    
    /**
     * Mueve al explorador un paso hacia arriba, incluye animacion.
     */
    public void moveUp()
    {
        int img=0;
        if(sound) Greenfoot.playSound(soundW[Greenfoot.getRandomNumber(2)]);
        for(int i=0; i<(60/animPixel); i++)
        {
            setLocation(getX(),getY()-animPixel);
            setImage(imgU[img++]);
            if(img==8)  img = 0;
            Greenfoot.delay(animDelay);
        }
        setImage(imgU[7]);        
    }
    
    /**
     * Mueve al explorador un paso hacia la derecha, incluye animacion.
     */
    public void moveRight()
    {
        int img=0;
        if(sound) Greenfoot.playSound(soundW[Greenfoot.getRandomNumber(2)]);
        for(int i=0; i<(60/animPixel); i++)
        {
            setLocation(getX()+animPixel,getY());
            setImage(imgR[img++]);
            if(img==8)  img = 0;
            Greenfoot.delay(animDelay);
        }
        setImage(imgR[7]);
    }
    
    /**
     * Mueve al explorador un paso hacia la izquierda, incluye animacion.
     */
    public void moveLeft()
    {
        int img=0;
        if(sound) Greenfoot.playSound(soundW[Greenfoot.getRandomNumber(2)]);
        for(int i=0; i<(60/animPixel); i++)
        {
            setLocation(getX()-animPixel,getY());
            setImage(imgL[img++]);
            if(img==8)  img = 0;
            Greenfoot.delay(animDelay);
        }
        setImage(imgL[7]);
    }
    
    /**
     * Regresa a la posicion original.
     */
    public void reset()
    {
        int x = stepsX.get(stepsX.size()-1);
        int y = stepsY.get(stepsY.size()-1);
        
        stepsX.clear();
        stepsY.clear();
        
        stepsX.add(0,x);
        stepsY.add(0,y);
        
        setLocation(243+(x*60), 110+(y*60));
        setImage(imgD[3]);
        active = true;
    }
    
    /**
     * Regresa a un estado anterior.
     */
    public void undo()
    {
        if(stepsX.size() > 1)
            stepsX.remove(0);
        if(stepsY.size() > 1)
            stepsY.remove(0);
        
        setLocation(243+(stepsX.get(0)*60), 110+(stepsY.get(0)*60));
        setImage(imgD[3]);
        active = true;
    }
    
    /**
     * Regresa la coordenada del explorador con respecto al tablero en X.
     * @return coordenada en X del explorador con respecto al tablero.
     */
    public int x()
    {
        return stepsX.get(0);
    }
    
    /**
     * Regresa la coordenada del explorador con respecto al tablero en Y
     * @return coordenada en Y del explorador con respecto al tablero.
     */
    public int y()
    {
        return stepsY.get(0);
    }
    
    /**
     * Pone una imagen transparente al explorador para ocultarlo.
     * Desactiva el movimiento del explorador.
     */
    public void invisible()
    {
        setImage("expInvisible.png");
        active = false;
        if(sound) Greenfoot.playSound("poison.wav");
    }
}
