import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.ArrayList;

/**
 * Clase que representa a la momia, controla las animaciones y su movimiento.
 * 
 * @author Jaime Vazquez Castillo 
 * @version 07082010
 */
public class Mummy  extends Enemy
{

    private static String[] imgU = {"mumUR0.png", "mumUR1.png", "mumUR0.png",
        "mumUN.png", "mumUL0.png", "mumUL1.png", "mumUL0.png", "mumUN.png"};
    private static String[] imgD = {"mumDR0.png", "mumDR1.png", "mumDR0.png",
        "mumDN.png", "mumDL0.png", "mumDL1.png", "mumDL0.png", "mumDN.png"};
    private static String[] imgR = {"mumRR0.png", "mumRR1.png", "mumRR0.png",
        "mumRN.png", "mumRL0.png", "mumRL1.png", "mumRL0.png", "mumRN.png"};
    private static String[] imgL = {"mumLR0.png", "mumLR1.png", "mumLR0.png",
        "mumLN.png", "mumLL0.png", "mumLL1.png", "mumLL0.png", "mumLN.png"};
    private static String[] imgF = {"f0.png", "f1.png", "f2.png", "f3.png",
        "f4.png", "f5.png", "f6.png", "f7.png", "f8.png", "f9.png", "f10.png",
        "f11.png", "f12.png", "f13.png", "f14.png", "f15.png", "f16.png", "f17.png",
        "f18.png", "f19.png", "f20.png", "f21.png", "f22.png", "f23.png", "f24.png",
        "f25.png", "f26.png", "f27.png", "f28.png", "f29.png", "f30.png", "f31.png",};
    private static String[] soundW = {"mumwalka.wav", "mumwalkb.wav"};
    
    private static int animDelay = 2;
    private static int animPixel = 4;
    private int idleCount;
    private boolean sound = true;
    
    private ArrayList<Integer> stepsX;
    private ArrayList<Integer> stepsY;
    private int[][] walls;
    private Explorer exp;
    
    /**
     * Constructor con el valor de la ubicacion inicial de la momia.
     * @param x coordenada en X dentro del tablero.
     * @param y coordenada en Y dentro del tablero.
     * @param w arreglo bidimensionar con la informacion acerca de las paredes.
     * @param e referencia al explorador (jugador).
     */
    public Mummy(int x, int y, int[][] w, Explorer e)
    {
        stepsX = new ArrayList<Integer>();
        stepsY = new ArrayList<Integer>();
        stepsX.add(0,x);
        stepsY.add(0,y);
        walls = w;
        exp = e;
        idleCount = 300;
        if(sound) Greenfoot.playSound("mummyhowl.wav");
    }
    /**
     * Act - do whatever the Mummy wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        if(idleCount<=37)
        {
            setImage("dance" + idleCount + ".png");
            Greenfoot.delay(2);
        }
        if(idleCount>=500) idleCount = 0;
        idleCount++;
        
    }    
 
    /**
     * Animacion de baile para la momia.
     */
    private void dance()
    {//0 - 13 - 8 - 30 - 24 - 37
        setImage("mumDN.png");
        int i = 0;
        int delay = 2;
        while(i < 13)
        {
            setImage("dance"+i+".png");
            Greenfoot.delay(delay);
            i++;
        }
        while(i > 8)
        {
            setImage("dance"+i+".png");
            Greenfoot.delay(delay);
            i--;
        }
        while(i < 30)
        {
            setImage("dance"+i+".png");
            Greenfoot.delay(delay);
            i++;
        }
        while(i > 24)
        {
            setImage("dance"+i+".png");
            Greenfoot.delay(delay);
            i--;
        }
        while(i <= 37)
        {
            setImage("dance"+i+".png");
            Greenfoot.delay(delay);
            i++;
        }
        setImage("mumDN.png");
    }
    
    
    /**
     * Activa el movimiento de la momia a 2 pasos.
     */
    public void move()
    {
        int steps = 0;
        idleCount = 300;
        while(!catchedHim() && steps < 2)
        {
            if(exp.x() > x() && canMoveRight())//mueve a la derecha
            {
                moveRight();
                stepsX.add(0,x()+1);
                stepsY.add(0,y());
            }
            else if(exp.x() < x() && canMoveLeft())//mueve a la izquierda
            {
                moveLeft();
                stepsX.add(0,x()-1);
                stepsY.add(0,y());
            }
            else if(exp.y() > y() && canMoveDown())//mueve hacia abajo
            {
                moveDown();   
                stepsX.add(0,x());
                stepsY.add(0,y()+1);
            }
            else if(exp.y() < y() && canMoveUp())//mueve hacia arriba
            {
                moveUp();
                stepsX.add(0,x());
                stepsY.add(0,y()-1);
            }else
            {
                stepsX.add(0,x());
                stepsY.add(0,y());
            }
            steps++;
        }
    }
    
    /**
     * Verifica si la momia puede moverse hacia abajo.
     * Codificacion de paredes:
     * 1 = pared este : 2 = pared sur : 3 = ambas
     * @return verdadero o false segun pueda moverse.
     */
    private boolean canMoveDown()
    {
        if(y() < 5 ) //si no esta en el borde de abajo...
        if(walls[x()][y()] != 2 && walls[x()][y()] != 3) //si no hay paredes que impidan...
            return true;
        
        return false;
    }
    
    /**
     * Verifica si la momia puede moverse hacia arriba.
     * Codificacion de paredes:
     * 1 = pared este : 2 = pared sur : 3 = ambas
     * @return verdadero o false segun pueda moverse.
     */
    private boolean canMoveUp()
    {
        if(y() > 0 ) //si no esta en el borde de arriba...
        if(walls[x()][y()-1] != 2 && walls[x()][y()-1] != 3) //si no hay paredes que impidan...
            return true;
        
        return false;
    }
    
    /**
     * Verifica si la momia puede moverse hacia la derecha.
     * Codificacion de paredes:
     * 1 = pared este : 2 = pared sur : 3 = ambas
     * @return verdadero o false segun pueda moverse.
     */
    private boolean canMoveRight()
    {
        if(x() < 5 ) //si no esta en el borde de la derecha...
        if(walls[x()][y()] != 1 && walls[x()][y()] != 3) //si no hay paredes que impidan...
            return true;
        
        return false;
    }
    
    /**
     * Verifica si la momia puede moverse hacia la derecha.
     * Codificacion de paredes:
     * 1 = pared este : 2 = pared sur : 3 = ambas
     * @return verdadero o false segun pueda moverse.
     */
    private boolean canMoveLeft()
    {
        if(x() > 0 ) //si no esta en el borde de la izquierda...
        if(walls[x()-1][y()] != 1 && walls[x()-1][y()] != 3) //si no hay paredes que impidan...
            return true;
        
        return false;
    }
    
    /**
     * Pregunta si ya atrapo al explorador. Le ordena desactivarse si es verdadero.
     * @return verdadero o falso segun lo atrape.
     */
    private boolean catchedHim()
    {
        Explorer explorer = (Explorer) getOneObjectAtOffset(0, 0, Explorer.class);
        if(explorer != null) {
            explorer.invisible();
            mummyWon();
            return true;
        }
        return false;
    }
    
    /**
     * Mueve a la mommia un paso hacia abajo, incluye animacion.
     */
    private void moveDown()
    {
        int img=0;
        if(sound) Greenfoot.playSound(soundW[Greenfoot.getRandomNumber(2)]);
        for(int i=0; i<(60/animPixel); i++)
        {
            setLocation(getX(),getY()+animPixel);
            setImage(imgD[img++]);
            if(img==8) img = 0;
            Greenfoot.delay(animDelay);
        }
        setImage(imgD[7]);
    }
    
    /**
     * Mueve a la momia un paso hacia arriba, incluye animacion.
     */
    private void moveUp()
    {
        int img=0;
        if(sound) Greenfoot.playSound(soundW[Greenfoot.getRandomNumber(2)]);
        for(int i=0; i<(60/animPixel); i++)
        {
            setLocation(getX(),getY()-animPixel);
            setImage(imgU[img++]);
            if(img==8)  img = 0;
            Greenfoot.delay(animDelay);
        }
        setImage(imgU[7]);        
    }
    
    /**
     * Mueve a la momia un paso hacia la derecha, incluye animacion.
     */
    private void moveRight()
    {
        int img=0;
        if(sound) Greenfoot.playSound(soundW[Greenfoot.getRandomNumber(2)]);
        for(int i=0; i<(60/animPixel); i++)
        {
            setLocation(getX()+animPixel,getY());
            setImage(imgR[img++]);
            if(img==8)  img = 0;
            Greenfoot.delay(animDelay);
        }
        setImage(imgR[7]);
    }
    
    /**
     * Mueve a la momia un paso hacia la izquierda, incluye animacion.
     */
    private void moveLeft()
    {
        int img=0;
        if(sound) Greenfoot.playSound(soundW[Greenfoot.getRandomNumber(2)]);
        for(int i=0; i<(60/animPixel); i++)
        {
            setLocation(getX()-animPixel,getY());
            setImage(imgL[img++]);
            if(img==8)  img = 0;
            Greenfoot.delay(animDelay);
        }
        setImage(imgL[7]);
    }
    
    /**
     * Animacion para cuando se atrapa al explorador.
     */
    private void mummyWon()
    {
        Greenfoot.playSound("pummel.wav");
        for(int i=0; i<32; i++)
        {
            setImage(imgF[i]);
            Greenfoot.delay(2);
        }
        setImage("mummyWin.png");
        Greenfoot.delay(1);
    }
    
    /**
     * Regresa a la posicion original del laberinto.
     */
    public void reset()
    {
        int x = stepsX.get(stepsX.size()-1);
        int y = stepsY.get(stepsY.size()-1);
        
        stepsX.clear();
        stepsY.clear();
        
        stepsX.add(0,x);
        stepsY.add(0,y);
        
        setLocation(243+(x*60), 110+(y*60));
        setImage(imgD[3]);
    }
    
    /**
     * Regresa a una posicion anterior si esta existe.
     */
    public void undo()
    {
        if(stepsX.size() > 1)
            stepsX.remove(0);
        if(stepsY.size() > 1)
            stepsY.remove(0);
        if(stepsX.size() > 1)
            stepsX.remove(0);
        if(stepsY.size() > 1)
            stepsY.remove(0);
        
        setLocation(243+(stepsX.get(0)*60), 110+(stepsY.get(0)*60));
        setImage(imgD[3]);
    }
    
    /**
     * Regresa la coordenada de la momia con respecto al tablero en X
     * @return coordenada en X de la momia con respecto al tablero.
     */
    public int x()
    {
        return stepsX.get(0);
    }
    
    /**
     * Regresa la coordenada de la momia con respecto al tablero en Y
     * @return coordenada en Y de la momia con respecto al tablero.
     */
    public int y()
    {
        return stepsY.get(0);
    }
}
